#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
Tests for the functionality of the Notification REST API.
The logic can be found in `notification/views/notify/py`.
"""
import http
from unittest.mock import patch

import notification.views.notify
import pytest
from pyramid.testing import DummyRequest

from workspaces.notification.services.notification_service import NotificationService


@pytest.mark.usefixtures("dummy_send", "mock_notification_service")
class TestNotifyViews:
    def test_view_notifications(self, dummy_send: DummyRequest):
        from notification.views.notify import NotificationListRestService

        response = NotificationListRestService(dummy_send).list_notification_templates()
        assert len(response) == 1

    def test_send_notification(self, dummy_send: DummyRequest, mock_notification_service: NotificationService):
        from notification.views.notify import NotificationRestService

        dummy_send.json_body = {
            "destination_email": "chausman@nrao.edu",
            "subject": "first thing",
            "message": "second thing",
        }

        dummy_send.matchdict["template_name"] = "email"
        with patch(
            "workspaces.notification.services.notification_service.NotificationService.send_email"
        ) as send_email:
            send_email.return_value = {}
            response = NotificationRestService(dummy_send).send_notification()
        assert response["errors"] == {}
        assert response["message"] == "Email sent"

    def test_create_notification(self, dummy_send: DummyRequest):
        from notification.views.notify import NotificationListRestService

        dummy_send.json_body = {
            "name": "new_template",
            "description": "a test template",
            "content": "Subject: {{foo}}\n\n{{bar}}",
        }

        response = NotificationListRestService(dummy_send).create_notification_template()
        assert response.status_code == http.HTTPStatus.CREATED
        assert response.json_body == dummy_send.json_body

    def test_delete_notification(self, dummy_send: DummyRequest):
        from notification.views.notify import NotificationRestService

        dummy_send.matchdict["template_name"] = "new_template"

        response = NotificationRestService(dummy_send).delete_notification_template()
        assert response.status_code == http.HTTPStatus.OK
